@@ -1,11 +1,14 @@
-from forms import Form
+from __future__ import absolute_import
+
 from django.core.exceptions import ValidationError
+from django.forms import Form
+from django.forms.fields import IntegerField, BooleanField
+from django.forms.util import ErrorList
+from django.forms.widgets import Media, HiddenInput
 from django.utils.encoding import StrAndUnicode
 from django.utils.safestring import mark_safe
 from django.utils.translation import ugettext as _
-from fields import IntegerField, BooleanField
-from widgets import Media, HiddenInput
-from util import ErrorList
+
 
 __all__ = ('BaseFormSet', 'all_valid')
 
@@ -58,11 +61,15 @@ class BaseFormSet(StrAndUnicode):
 
     def __getitem__(self, index):
         """Returns the form at the given index, based on the rendering order"""
-        return list(self)[index]
+        return self.forms[index]
 
     def __len__(self):
         return len(self.forms)
 
+    def __nonzero__(self):
+        """All formsets have a management form which is not included in the length"""
+        return True
+
     def _management_form(self):
         """Returns the ManagementForm instance for this FormSet."""
         if self.is_bound:
@@ -118,7 +125,7 @@ class BaseFormSet(StrAndUnicode):
         if self.is_bound:
             defaults['data'] = self.data
             defaults['files'] = self.files
-        if self.initial:
+        if self.initial and not 'initial' in kwargs:
             try:
                 defaults['initial'] = self.initial[i]
             except IndexError:
@@ -216,15 +223,14 @@ class BaseFormSet(StrAndUnicode):
                     return (1, 0) # +infinity, larger than any number
                 return (0, k[1])
             self._ordering.sort(key=compare_ordering_key)
-        # Return a list of form.cleaned_data dicts in the order spcified by
+        # Return a list of form.cleaned_data dicts in the order specified by
         # the form data.
         return [self.forms[i[0]] for i in self._ordering]
     ordered_forms = property(_get_ordered_forms)
 
-    #@classmethod
+    @classmethod
     def get_default_prefix(cls):
         return 'form'
-    get_default_prefix = classmethod(get_default_prefix)
 
     def non_form_errors(self):
         """
@@ -300,6 +306,12 @@ class BaseFormSet(StrAndUnicode):
         """
         pass
 
+    def has_changed(self):
+        """
+        Returns true if data in any form differs from initial.
+        """
+        return any(form.has_changed() for form in self)
+
     def add_fields(self, form, index):
         """A hook for adding extra fields on to each form instance."""
         if self.can_order:
@@ -316,7 +328,7 @@ class BaseFormSet(StrAndUnicode):
 
     def is_multipart(self):
         """
-        Returns True if the formset needs to be multipart-encrypted, i.e. it
+        Returns True if the formset needs to be multipart, i.e. it
         has FileInput. Otherwise, False.
         """
         return self.forms and self.forms[0].is_multipart()
